@@ -4,18 +4,24 @@
 # XML::Edifact is free software. You can redistribute and/or
 # modify this copy under terms of GNU General Public License.
 #
-# This is a 0.30 version: Anything is still in flux.
+# This is a 0.32 version: Anything is still in flux.
 # DO NOT EXPECT FURTHER VERSION TO BE COMPATIBLE!
 
 package XML::Edifact;
 
 use strict;
+use SDBM_File;
+use Fcntl;
+use XML::Parser;
+use Carp;
+
 use vars qw($VERSION $debug $local_url);
 
 $VERSION='0.30';
 $local_url='.';					# edit your local url here
 $debug=1;					# debug=1 is fine
 
+# ------------------------------------------------------------------------------
 # edit the HERE documents for those variables for your systems preferences.
 # Ive included both kinds of namespace definitions. You may have to drop one
 # if your system is aware about xml namespaces.
@@ -27,49 +33,36 @@ use vars qw(
 	$COMPOSITE_SPECIFICATION_HEADER
 	);
 
-# ------------------------------------------------------------------------------
 $MESSAGE_HEADER=<<HERE_MESSAGE_HEADER;
 <?xml version="1.0"?>
-
-<?xml:namespace ns='$local_url/edicooked03.rdf'		prefix='edicooked' ?>
-<?xml:namespace ns='$local_url/edicooked03_trsd.rdf'	prefix='trsd' ?>
-<?xml:namespace ns='$local_url/edicooked03_trcd.rdf'	prefix='trcd' ?>
-<?xml:namespace ns='$local_url/edicooked03_tred.rdf'	prefix='tred' ?>
-<?xml:namespace ns='$local_url/edicooked03_uncl.rdf'	prefix='uncl' ?>
-<?xml:namespace ns='$local_url/edicooked03_anxs.rdf'	prefix='anxs' ?>
-<?xml:namespace ns='$local_url/edicooked03_anxc.rdf'	prefix='anxc' ?>
-<?xml:namespace ns='$local_url/edicooked03_anxe.rdf'	prefix='anxe' ?>
-<?xml:namespace ns='$local_url/edicooked03_unsl.rdf'	prefix='unsl' ?>
-<?xml:namespace ns='$local_url/edicooked03_unknown.rdf'	prefix='unknown' ?>
-
-<!DOCTYPE edicooked:message SYSTEM "$local_url/edicooked03.dtd">
+<!DOCTYPE edifact:message SYSTEM "$local_url/edifact03.dtd">
 
 <!-- XML message produced by edi2xml.pl (c) Kraehe\@Bakunin.North.De -->
 
-<edicooked:message
-	xmlns:edicooked='$local_url/edicooked03.rdf' 
-	xmlns:trsd='$local_url/edicooked03_trsd.rdf'
-	xmlns:trcd='$local_url/edicooked03_trcd.rdf'
-	xmlns:tred='$local_url/edicooked03_tred.rdf'
-	xmlns:uncl='$local_url/edicooked03_uncl.rdf'
-	xmlns:anxs='$local_url/edicooked03_anxe.rdf'
-	xmlns:anxc='$local_url/edicooked03_anxc.rdf'
-	xmlns:anxe='$local_url/edicooked03_anxe.rdf'
-	xmlns:unsl='$local_url/edicooked03_unsl.rdf'
-	xmlns:unknown='$local_url/edicooked03_unknown.rdf'
+<edifact:message
+	xmlns:edifact='$local_url/edifact03.rdf' 
+	xmlns:trsd='$local_url/edifact03_trsd.rdf'
+	xmlns:trcd='$local_url/edifact03_trcd.rdf'
+	xmlns:tred='$local_url/edifact03_tred.rdf'
+	xmlns:uncl='$local_url/edifact03_uncl.rdf'
+	xmlns:anxs='$local_url/edifact03_anxe.rdf'
+	xmlns:anxc='$local_url/edifact03_anxc.rdf'
+	xmlns:anxe='$local_url/edifact03_anxe.rdf'
+	xmlns:unsl='$local_url/edifact03_unsl.rdf'
+	xmlns:unknown='$local_url/edifact03_unknown.rdf'
 	>
 HERE_MESSAGE_HEADER
 # ------------------------------------------------------------------------------
 $DOCTYPE_HEADER=<<HERE_DOCTYPE_HEADER;
 <!-- XML DTD for cooked EDI to reflect raw UN/EDIFACT -->
-<!-- edicooked03.dtd (c) '98 Kraehe\@Bakunin.North.De -->
+<!-- edifact03.dtd (c) '98 Kraehe\@Bakunin.North.De -->
 
 <!-- I should warn you that badly written validating XML
      parsers may have problems by running out of memory.
 
      Its quite large, I know, but it's not yet complete!
 
-     My first attempt on an automatic generated edicooked03.dtd
+     My first attempt on an automatic generated edifact03.dtd
      failed with massive "content model is ambiguous" errors.
      So I deceided to use a mixed content model to simplify work
      for XML parsers and also for me.
@@ -91,38 +84,36 @@ HERE_DOCTYPE_HEADER
 # ------------------------------------------------------------------------------
 $SEGMENT_SPECIFICATION_HEADER=<<HERE_SEGMENT_SPECIFICATION_HEADER;
 <?xml version="1.0"?>
-<?xml:namespace ns='$local_url/edicooked03.dtd'	prefix='edicooked' ?>
-<!DOCTYPE edicooked:segment_specification SYSTEM "edicooked03.dtd">
+<?xml:namespace ns='$local_url/edifact03.dtd'	prefix='edifact' ?>
+<!DOCTYPE edifact:segment_specification SYSTEM "edifact03.dtd">
 <!-- XML::Edifact segment.xml (c) Kraehe\@Bakunin.North.De -->
 
-<edicooked:segment_specifications
-	xmlns:edicooked='$local_url/edicooked03.dtd'
+<edifact:segment_specifications
+	xmlns:edifact='$local_url/edifact03.dtd'
 	>
 HERE_SEGMENT_SPECIFICATION_HEADER
 # ------------------------------------------------------------------------------
 $COMPOSITE_SPECIFICATION_HEADER=<<HERE_COMPOSITE_SPECIFICATION_HEADER;
 <?xml version="1.0"?>
-<?xml:namespace ns='$local_url/edicooked03.dtd'	prefix='edicooked' ?>
-<!DOCTYPE edicooked:composite_specification SYSTEM "edicooked03.dtd">
+<?xml:namespace ns='$local_url/edifact03.dtd'	prefix='edifact' ?>
+<!DOCTYPE edifact:composite_specification SYSTEM "edifact03.dtd">
 <!-- XML::Edifact composite.xml (c) Kraehe\@Bakunin.North.De -->
 
-<edicooked:composite_specifications
-	xmlns:edicooked='$local_url/edicooked03.dtd'
+<edifact:composite_specifications
+	xmlns:edifact='$local_url/edifact03.dtd'
 	>
 HERE_COMPOSITE_SPECIFICATION_HEADER
 # ------------------------------------------------------------------------------
 
 use vars qw(%SEGMT %COMPT %CODET %ELEMT);
-use vars qw(%SEGMR %COMPR %CODER %ELEMR);
+use vars qw(%SEGMR);
 
-use vars qw($raw_message $advice $advice_component_seperator);
+use vars qw($edi_message $xml_message);
+use vars qw($advice $advice_component_seperator);
 use vars qw($advice_element_seperator $advice_decimal_notation);
 use vars qw($advice_release_indicator $advice_segment_terminator);
 
-use SDBM_File;
-use Fcntl;
-
-print "<!-- Hello from XML::EDIFACT -->\n"		if ($debug>1);
+$xml_message .= "<!-- Hello from XML::EDIFACT -->\n"		if ($debug>1);
 
 # ------------------------------------------------------------------------------
 
@@ -131,14 +122,12 @@ sub open_dbm {
 
     $fcntl = O_RDWR					unless $fcntl eq "";
 
-    printf "<!-- *** open_dbm -->\n"			if ($debug>1);
+    $xml_message .= sprintf "<!-- *** open_dbm -->\n"			if ($debug>1);
 
     tie(%SEGMT, 'SDBM_File', $directory.'/segment.dat',   $fcntl, 0644)	|| die "can not tie segment.dat:".$!;
     tie(%SEGMR, 'SDBM_File', $directory.'/segment.rev',   $fcntl, 0644)	|| die "can not tie segment.dat:".$!;
     tie(%COMPT, 'SDBM_File', $directory.'/composite.dat', $fcntl, 0644)	|| die "can not tie composite.dat:".$!;
-    tie(%COMPR, 'SDBM_File', $directory.'/composite.rev', $fcntl, 0644)	|| die "can not tie composite.dat:".$!;
     tie(%ELEMT, 'SDBM_File', $directory.'/element.dat',   $fcntl, 0644)	|| die "can not tie element.dat:".$!;
-    tie(%ELEMR, 'SDBM_File', $directory.'/element.rev',   $fcntl, 0644)	|| die "can not tie element.dat:".$!;
     tie(%CODET, 'SDBM_File', $directory.'/codes.dat',     $fcntl, 0644)	|| die "can not tie codes.dat:".$!;
 }
 
@@ -146,9 +135,7 @@ sub close_dbm {
     untie(%SEGMT);
     untie(%SEGMR);
     untie(%COMPT);
-    untie(%COMPR);
     untie(%ELEMT);
-    untie(%ELEMR);
     untie(%CODET);
 }
 
@@ -169,20 +156,22 @@ sub recode_mark {
 
 # -----------------------------------------------------------------------------
 
-sub read_message() {
+sub read_edi_message() {
 	my($filename) = @_;
 	my($size);
 
-	printf "<!-- *** reading message -->\n"		 if ($debug>1);
+	$xml_message .= sprintf "<!-- *** reading message -->\n"		 if ($debug>1);
 
 	$size=(stat($filename))[7]		|| die "cant stat ".$filename;
 	die $filename." is to short ".$size." for EDI" 	if ($size <= 9);
 	open(F,$filename)			|| die "cant open ".$filename;
-	read(F,$raw_message,$size,0)		|| die "cant read message from ".$filename;
+	read(F,$edi_message,$size,0)		|| die "cant read message from ".$filename;
 	close(F);
 
-	$advice=substr($raw_message,0,9);
-	die $filename." is not an EDI message"	if ($advice !~ "^UNA");
+	$advice=substr($edi_message,0,9);
+	die $filename." is not an EDI message"	if ($advice !~ "^UN[AB]");
+
+	$advice = "UNA:+.? '" unless ($advice =~ "^UNA");
 
 	$advice_component_seperator =substr($advice,3,1);
 	$advice_element_seperator   =substr($advice,4,1);
@@ -191,25 +180,27 @@ sub read_message() {
 	$advice_segment_terminator  =substr($advice,8,1);
 }
 
-sub process_message() {
+sub make_xml_message() {
 
-	print $MESSAGE_HEADER;
+	$xml_message = $MESSAGE_HEADER;
 
 	my($cooked_message,@Segments,$segment,$s);
 
-	$cooked_message  = $raw_message;
+	$cooked_message  = $edi_message;
 	$s = "\\".$advice_release_indicator."\\".$advice_segment_terminator;
 	$cooked_message =~ s/$s/\001/g;
 
 	@Segments = split /$advice_segment_terminator/, $cooked_message;
-	shift @Segments;
+	shift @Segments if ($Segments[0] =~ "^UNA");
 
 	foreach $segment (@Segments) {
 		$s = "\\".$advice_segment_terminator;
 		$segment =~ s/\001/$s/g;
 		&resolve_segment($segment);
 	}
-	print "</edicooked:message>\n";
+	$xml_message .= "</edifact:message>\n";
+
+	return $xml_message;
 }
 
 # -----------------------------------------------------------------------------
@@ -226,7 +217,7 @@ sub resolve_segment {
 		if ($debug) {
 			$comment = $raw_segment;
 			$comment =~ s/--/__/g;
-			printf "<!-- SEGMENT %s -->\n\n", $comment;
+			$xml_message .= sprintf "<!-- SEGMENT %s -->\n\n", $comment;
 		}
 
 		$cooked_segment = $raw_segment;
@@ -238,9 +229,9 @@ sub resolve_segment {
 		@sgv = split("\t", $SEGMT{$Elements[0]}, 4);
 		@Codes    = split / /,  " ".$sgv[0];
 
-		printf "<!-- *** name %s %s -->\n", $Elements[0], $sgv[3]	if ($debug>2);
+		$xml_message .= sprintf "<!-- *** name %s %s -->\n", $Elements[0], $sgv[3]	if ($debug>2);
 		if (($sgv[2] ne '') && ($#Codes>=$#Elements)) {
-			printf "  <%s>\n", $sgv[2];
+			$xml_message .= sprintf "  <%s>\n", $sgv[2];
 
 			for ($i = 1; $i <= $#Elements; $i++) {
 				$element  = $Elements[$i];
@@ -248,13 +239,13 @@ sub resolve_segment {
 
 				if ($Elements[$i] ne '') {
 					# resolve_element
-					printf "<!-- *** %s='%s' -->\n", $Codes[$i],$Elements[$i] if ($debug>1);
+					$xml_message .= sprintf "<!-- *** %s='%s' -->\n", $Codes[$i],$Elements[$i] if ($debug>1);
 					resolve_element($Codes[$i], $Elements[$i]);
 				}
 			}
-			printf "  </%s>\n\n", $sgv[2];
+			$xml_message .= sprintf "  </%s>\n\n", $sgv[2];
 		} else {
-			printf "  <edicooked:raw_segment data=\"%s\"/>\n", $raw_segment;
+			$xml_message .= sprintf "  <edifact:raw_segment data=\"%s\"/>\n", $raw_segment;
 		}
 	}
 }
@@ -269,7 +260,7 @@ sub resolve_element {
 
 	$ok=0;
 
-	printf "<!-- *** resolve element %s %s -->\n", $code, $raw_element if ($debug>1);
+	$xml_message .= sprintf "<!-- *** resolve element %s %s -->\n", $code, $raw_element if ($debug>1);
 
 	$cooked_element = $raw_element;
 	$s = "\\".$advice_release_indicator."\\".$advice_component_seperator;
@@ -279,7 +270,7 @@ sub resolve_element {
 
 		@cmv = split("\t", $cm, 4);
 
-		printf "    <%s>\n", $cmv[2]			if ($cmv[2]);
+		$xml_message .= sprintf "    <%s>\n", $cmv[2]			if ($cmv[2]);
 
 		$s = "\\".$advice_component_seperator;
 		@Components = split /$s/, $cooked_element;
@@ -294,7 +285,7 @@ sub resolve_element {
 			}
 		}
 
-		printf "    </%s>\n", $cmv[2]			if ($cmv[2]);
+		$xml_message .= sprintf "    </%s>\n", $cmv[2]			if ($cmv[2]);
 		$ok=1;
 	}
 	if (($code =~ "^[0-9]") && ($cm = $ELEMT{$code})) {
@@ -303,7 +294,7 @@ sub resolve_element {
 		&resolve_code($code, $cooked_element);
 		$ok=1;
 	}
-	printf "  <edicooked:raw_element data=\"%s\"/>\n", $raw_element unless $ok;
+	$xml_message .= sprintf "  <edifact:raw_element data=\"%s\"/>\n", $raw_element unless $ok;
 }
 
 # -----------------------------------------------------------------------------
@@ -313,20 +304,215 @@ sub resolve_code {
 	my($cd,$mark,@cdv);
 
 	$mark=$ELEMT{$code};
-	printf "<!-- *** resolve code %s %s -->\n", $code, $val if ($debug>1);
+	$xml_message .= sprintf "<!-- *** resolve code %s %s -->\n", $code, $val if ($debug>1);
 
 	if ($CODET{$code."\t"} ne "") {
 		$cd = $CODET{$code."\t".$val};
 		if ($cd ne '') {
 			@cdv=split /\t/, $cd;
-			printf "      <%s %s:code=\"%s:%s\">%s</%s>\n", $mark, $cdv[0], $code, $val, $cdv[1], $mark;
+			$xml_message .= sprintf "      <%s %s:code=\"%s:%s\">%s</%s>\n", $mark, $cdv[0], $code, $val, $cdv[1], $mark;
 		}
 		else {
-			printf "      <%s unknown:code=\"%s:%s\">%s</%s>\n", $mark, $code, $val, $val, $mark;
+			$xml_message .= sprintf "      <%s unknown:code=\"%s:%s\">%s</%s>\n", $mark, $code, $val, $val, $mark;
 		}
 	}
 	else {
-		printf "      <%s>%s</%s>\n", $mark, $val, $mark;
+		$xml_message .= sprintf "      <%s>%s</%s>\n", $mark, $val, $mark;
+	}
+}
+
+# -----------------------------------------------------------------------------
+
+sub read_xml_message() {
+	my($filename) = @_;
+	my($size);
+
+	$size=(stat($filename))[7]		|| die "cant stat ".$filename;
+	die $filename." is to short ".$size." for EDI" 	if ($size <= 9);
+	open(F,$filename)			|| die "cant open ".$filename;
+	read(F,$xml_message,$size,0)		|| die "cant read message from ".$filename;
+	close(F);
+
+	$advice_component_seperator = ":";
+	$advice_element_seperator   = "+";
+	$advice_decimal_notation    = ".";
+	$advice_release_indicator   = "?";
+	$advice_segment_terminator  = "'";
+}
+
+use vars qw(@edi_segment @edi_group $edi_valid $edi_level $edi_si $edi_gi);
+
+sub make_edi_message() {
+	my $xml_parser;
+
+	$edi_message = "UNA";
+	$edi_message .= $advice_component_seperator;
+	$edi_message .= $advice_element_seperator;
+	$edi_message .= $advice_decimal_notation;
+	$edi_message .= $advice_release_indicator;
+	$edi_message .= " ";
+	$edi_message .= $advice_segment_terminator;
+
+	$edi_level    = 0;
+
+	$xml_parser   = new XML::Parser(Handlers => {	Start => \&handle_start,
+							End   => \&handle_end,
+							Char  => \&handle_char});
+	$xml_parser -> parse($xml_message);
+
+	return $edi_message;
+}
+
+sub handle_start {
+	my $expat   = shift @_;
+	my $element = shift @_;
+	my %options = @_;
+	my ($opt,$val,$i);
+	my (@sgv,@cmv,@sgc,@cmc,$junk,$trans);
+
+	if ($debug>1) {
+		printf STDERR "(%s\n", $element;
+		foreach $opt (keys (%options)) {
+			printf STDERR "A%s=%s\n", $opt, $options{$opt};
+		}
+	}
+
+	if ($edi_level == 0) {
+		die "this is not XML::Edifact" if ($element !~ /^[^:]*:message/);
+	}
+	if ($edi_level == 1) {
+		if ($element =~ /^[^:]*:raw_segment/) {
+			foreach $opt (keys (%options)) {
+				if ($opt eq "data") {
+					$edi_message .= $options{$opt};
+					$edi_message .= $advice_segment_terminator;
+				}
+			}
+			@edi_segment = ();
+			$edi_si      = 0;
+			@edi_group   = ();
+			$edi_gi      = 0;
+			$edi_valid   = 1;
+		} else {
+			@edi_segment = ($SEGMR{$element});
+			$edi_si      = 0;
+			@edi_group   = ();
+			$edi_gi      = 0;
+			$edi_valid   = ($edi_segment[0] ne "");
+		}
+	}
+	if ($edi_valid) {
+	    if ($edi_level == 2) {
+		$edi_si++;
+		@edi_group   = ();
+		$edi_gi      = 0;
+
+		@sgv = split("\t", $SEGMT{$edi_segment[0]}, 4);
+		@sgc = split / /,  " ".$sgv[0];
+
+		SKIP_SEGMT: for ($i = $edi_si; $i <= $#sgc; $i++) {
+			if ($sgc[$i] =~ /^[A-Z]/) {
+				($junk, $junk, $trans, $junk) = split /\t/, $COMPT{$sgc[$i]};
+			} else {
+				$trans =$ELEMT{$sgc[$i]};
+			}
+			last SKIP_SEGMT if ($trans eq $element);
+		}
+
+		if ($i <= $#sgc) {
+			$edi_si = $i;
+		} else {
+			$edi_valid = 0;
+		}
+
+		foreach $opt (keys (%options)) {
+			if ($opt =~ "^[^:]*:code") {
+				$val = $options{$opt};
+				$val =~ s/^[^:]*://;
+				$edi_group[$edi_gi] = $val;
+			}
+		}
+	    }
+	    if ($edi_level == 3) {
+		@sgv = split("\t", $SEGMT{$edi_segment[0]}, 4);
+		@sgc = split / /,  " ".$sgv[0];
+		@cmv = split("\t", $COMPT{$sgc[$edi_si]}, 4);
+		@cmc = split / /,  $cmv[0];
+
+		SKIP_COMPT: for ($i = $edi_gi; $i <= $#cmc; $i++) {
+			$trans =$ELEMT{$cmc[$i]};
+			last SKIP_COMPT if ($trans eq $element);
+		}
+
+		if ($i <= $#cmc) {
+			$edi_gi = $i;
+		} else {
+			$edi_valid = 0;
+		}
+
+		foreach $opt (keys (%options)) {
+			if ($opt =~ "^[^:]*:code") {
+				$val = $options{$opt};
+				$val =~ s/^[^:]*://;
+				$edi_group[$edi_gi] = $val;
+			}
+		}
+	    }
+	}
+	$edi_level++;
+}
+
+sub handle_end {
+	my ($expat, $element) = @_;
+	my ($i,$cooked,$si,$s1,$s2);
+
+	if ($debug>1) {
+		printf STDERR ")%s\n", $element;
+	}
+
+	$edi_level--;
+
+	if ($edi_valid) {
+	    if ($edi_level == 1) {
+	    	if ($#edi_segment>0) {
+			$edi_message .= join $advice_element_seperator, @edi_segment;
+			$edi_message .= $advice_segment_terminator;
+		}
+	    }
+	    if ($edi_level == 2) {
+		for ($i = 0; $i<= $#edi_group; $i++) {
+			$cooked = $edi_group[$i];
+
+			foreach $si ($advice_release_indicator, $advice_component_seperator, $advice_element_seperator, $advice_segment_terminator) {
+				$s1 = "\\".$si;
+				$s2 = $advice_release_indicator.$si;
+				$cooked =~ s/$s1/$s2/g;
+			}
+
+			$edi_group[$i] = $cooked;
+		}
+		$edi_segment[$edi_si] .= join $advice_component_seperator, @edi_group;
+	    }
+	    if ($edi_level == 3) {
+		$edi_gi++;
+	    }
+	} else {
+	    carp "invalid message";
+	}
+}
+
+sub handle_char {
+	my ($expat, $element) = @_;
+
+
+	if ($element !~ /^[\n\r\t ]*$/) {
+		if ($debug>1) {
+			printf STDERR "-%s\n", $element;
+		}
+		$element =~ s/[\n\r\t ]*$//;
+		$element =~ s/^[\n\r\t ]*//;
+
+		$edi_group[$edi_gi] = $element if $edi_group[$edi_gi] eq "";
 	}
 }
 
@@ -341,13 +527,22 @@ XML::Edifact - Perl module to handle XML::Edifact messages.
 
 =head1 SYNOPSIS
 
-use XML::Edifact;
+use	XML::Edifact;
 
-&XML::Edifact::open_dbm("data");
-&XML::Edifact::read_message($ARGV[0]);
-&XML::Edifact::process_message();
-&XML::Edifact::close_dbm();
+	&XML::Edifact::open_dbm("data");
+	&XML::Edifact::read_edi_message($ARGV[0]);
+print	&XML::Edifact::make_xml_message();
+	&XML::Edifact::close_dbm();
+0;
 
+---------------------------------------------------------------
+
+use	XML::Edifact;
+
+	&XML::Edifact::open_dbm("data");
+	&XML::Edifact::read_xml_message($ARGV[0]);
+print	&XML::Edifact::make_edi_message();
+	&XML::Edifact::close_dbm();
 0;
 
 =head1 DESCRIPTION
@@ -362,7 +557,8 @@ EDIFACT message into a buffer global to the package, and
 to print this message as XML on STDOUT.
 
 The best think you can currently do with it:
-  Run the regession test found in ./bin/make_test.sh
+  Run the regession test found under Installation in
+  the README file, using you own files.
 
 The second best: Just view the files in ./examples with
 your favourite pager. Anything that can be found after
